import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Homevee-Cloud",
    version="0.0.0.13",
    author="Homevee-Cloud",
    author_email="support@homevee.de",
    description="Dein neues Smarthome-System!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/homevee/homevee-cloud",
    entry_points = {
        #'console_scripts': ['HomeveeCloud=HomeveeCloud.command_line:main'],
        'console_scripts': ['HomeveeCloudLegacy=HomeveeCloudLegacy:main'],
    },
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'pyfcm',
    ]
)

#Commands to build
#sudo apt-get install python3-pip python3-dev libffi-dev libssl-dev libxml2-dev libxslt1-dev libjpeg8-dev zlib1g-dev -y
#pip install --upgrade setuptools wheel twine
